///////////////////////////////////////////////////////////////////////////////
//                                                               
//                                                                
//  Module      : SelectMAP.v                                   
//  Last Update : 08/29/2001                                       
//                                                                 
//  Description : CPLD design for SelectMAP interface to microprocessor
//                                                                  
//                                                                   
//  Designer    : Mark Ng                                       
//  Company     : Xilinx Inc.                                       
//                                                                     
//                                                                     
//  Disclaimer  : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY  
//                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY     
//                IMPLIED WARRANTIES OF MERCHANTABILITY & FITNESS FOR  
//                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.      
//                THEY ARE ONLY INTENDED TO BE USED BY XILINX          
//                CUSTOMERS, AND WITHIN XILINX DEVICES.               
//                                                                     
//                Copyright (c) 2000 Xilinx, Inc.                      
//                All rights reserved                                  
//                                                                     
//                                                                      
//                                                                 
///////////////////////////////////////////////////////////////////////////////

module SelectMAP(
  ADDR,DATA,
  WEn,
  OEn,
  CS1n,
  Virtex_CCLK,
  Virtex_WRITE,
  Virtex_PROGRAM,
  Virtex_BUSY,
  Virtex_DONE,
  Virtex_INIT,
  Virtex_CS,
  Virtex_D7,
  Virtex_D6,
  Virtex_D5,
  Virtex_D4,
  Virtex_D3,
  Virtex_D2,
  Virtex_D1,
  Virtex_D0);

	//Original Spingboard Signals:

  input [23:0] ADDR;
  inout [15:0] DATA;
  input WEn;
  input OEn;
  input CS1n;

	//Virtex Configuration Signals:

  output Virtex_CCLK;
  output Virtex_WRITE;
  output Virtex_PROGRAM;
  input Virtex_BUSY;
  input Virtex_DONE;
  input Virtex_INIT;
  output Virtex_CS;
  output Virtex_D7;
  output Virtex_D6;
  output Virtex_D5;
  output Virtex_D4;
  output Virtex_D3;
  output Virtex_D2;
  output Virtex_D1;
  output Virtex_D0;

  reg [23:0] Configuration_Register_ADDR;
	reg [23:0] Program_Register_ADDR;
	reg [23:0] Input_Register_ADDR;
	 
	wire Output_Buffer_Enable; //Output Buffer Enable Signal
	 
	reg Input_Reg_D2;
	reg Input_Reg_D1;
	reg Input_Reg_D0;

	//Internal Wire and Regs
	reg [15:0] Input_Register_Data;
	reg Virtex_D7;
  reg Virtex_D6;
  reg Virtex_D5;
  reg Virtex_D4;
  reg Virtex_D3;
  reg Virtex_D2;
  reg Virtex_D1;
  reg Virtex_D0;
  reg Virtex_CCLK;
	reg Virtex_CS;
	reg Virtex_WRITE;
	reg Virtex_PROGRAM_OE;

	assign Configuration_Register_ADDR = 24'h0;
	assign Program_Register_ADDR       = 24'h2;
	assign Input_Register_ADDR         = 24'h4;
	
	assign Output_Buffer_Enable = !OEn & !CS1n;

	assign DATA = Output_Buffer_Enable ? Input_Register_Data : 16'hZ;
  
  //Configuration Register

	always @(posedge WEn)
		begin
			if (!CS1n)
			  begin
				  if (ADDR == Configuration_Register_ADDR)
					  begin
	  				  Virtex_D7 = DATA[15];
		          Virtex_D6 = DATA[14];
							Virtex_D5 = DATA[13];
						  Virtex_D4 = DATA[12];
							Virtex_D3 = DATA[11];
							Virtex_D2 = DATA[10];
							Virtex_D1 = DATA[9];
							Virtex_D0 = DATA[8];
							Virtex_CCLK = DATA[0];
						end
				end
		end

  //Program Register
	
	always @(WEn)
	   begin
		   if (!CS1n)
			   begin 
				   if (ADDR == Program_Register_ADDR)
					   begin
						   Virtex_CS = DATA[2];
	 						 Virtex_WRITE = DATA[1];
							 Virtex_PROGRAM_OE = DATA[0];
						 end
				 end
		 end
			 
	assign Virtex_PROGRAM = Virtex_PROGRAM_OE ? 1'b0 : 1'bZ;

  //Input Register

	always @(negedge OEn)
		begin
		  if (CS1n) 
			  begin
		      if (ADDR == Input_Register_ADDR)
			      begin
				      Input_Reg_D2 = Virtex_BUSY;
					    Input_Reg_D1 = Virtex_DONE;
					    Input_Reg_D0 = Virtex_INIT;
				    end
		    end
		end

	assign Input_Register_Data = {13'b0, Input_Reg_D2, Input_Reg_D1, Input_Reg_D0};
	
endmodule
