///////////////////////////////////////////////////////////////////////////////
//                                                               
//                                                                
//  Module      : SlaveSerial.v                                   
//  Last Update : 08/29/2001                                       
//                                                                 
//  Description : CPLD design for Slave Serial interface to microprocessor
//                                                                  
//                                                                   
//  Designer    : Mark Ng                                       
//  Company     : Xilinx Inc.                                       
//                                                                     
//                                                                     
//  Disclaimer  : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY  
//                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY     
//                IMPLIED WARRANTIES OF MERCHANTABILITY & FITNESS FOR  
//                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.      
//                THEY ARE ONLY INTENDED TO BE USED BY XILINX          
//                CUSTOMERS, AND WITHIN XILINX DEVICES.               
//                                                                     
//                Copyright (c) 2000 Xilinx, Inc.                      
//                All rights reserved                                  
//                                                                     
//                                                                      
//                                                                 
///////////////////////////////////////////////////////////////////////////////

module SlaveSerial(
  ADDR,DATA,
  WEn,
  OEn,
  CS1n,
  Virtex_CCLK,
  Virtex_PROGRAM,
  Virtex_DONE,
  Virtex_INIT,
  Virtex_Din
  );
    
	// Original Springboard Signals:
	input [23:0] ADDR;
  inout [15:0] DATA;
  input WEn;
  input OEn;
  input CS1n;
	
	// Virtex Configuration Signals:
  output Virtex_CCLK;
  output Virtex_PROGRAM;
  input  Virtex_DONE;
  input  Virtex_INIT;
  output Virtex_Din;

	reg [23:0] Configuration_Register_ADDR;
	reg [23:0] Program_Register_ADDR;
	reg [23:0] Input_Register_ADDR;
	 
	wire Output_Buffer_Enable; //Output Buffer Enable Signal
	
	reg Input_Reg_D1;
	reg Input_Reg_D0;

	//Internal Wire and Regs
	reg [15:0] Input_Register_Data;
  reg Virtex_CCLK;
	reg Virtex_PROGRAM_OE;
	reg Virtex_Din;

	assign Configuration_Register_ADDR = 24'h0;
  assign Program_Register_ADDR       = 24'h2;
	assign Input_Register_ADDR         = 24'h4;
	 
	assign Output_Buffer_Enable = !OEn & !CS1n;

	assign DATA = Output_Buffer_Enable ? Input_Register_Data : 16'hZ;
  
  //Configuration Register

	always @(posedge WEn)
	  begin
		  if (!CS1n)
		    begin
			    if (ADDR == Configuration_Register_ADDR)
			      begin
				      Virtex_CCLK = DATA[1];
				      Virtex_Din = DATA[0];
			      end
		    end
	  end

  //Program Register
	
	always @(WEn)
	  begin
		  if (!CS1n)
			  begin 
				  if (ADDR == Program_Register_ADDR)
					  begin
							Virtex_PROGRAM_OE = DATA[0];
					  end
			  end
		end
			 
	assign Virtex_PROGRAM = Virtex_PROGRAM_OE ? 1'b0 : 1'bZ;

  //Input Register

	always @(negedge OEn)
		begin
		   if (CS1n) 
			   begin
		         if (ADDR == Input_Register_ADDR)
			         begin
					      Input_Reg_D1 = Virtex_DONE;
					      Input_Reg_D0 = Virtex_INIT;
				      end
		      end
		end

	assign Input_Register_Data = {14'b0, Input_Reg_D1, Input_Reg_D0};
      
endmodule
