-------------------------------------------------------------------------------
--                                                               
--                                                                
--  Module      : SlaveSerial.vhd                                   
--  Last Update : 08/29/2001                                       
--                                                                 
--  Description : CPLD design for Slave Serial interface to microprocessor
--                                                                  
--                                                                   
--  Designer    : Mark Ng                                       
--  Company     : Xilinx Inc.                                       
--                                                                     
--                                                                     
--  Disclaimer  : THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY  
--                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY     
--                IMPLIED WARRANTIES OF MERCHANTABILITY & FITNESS FOR  
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.      
--                THEY ARE ONLY INTENDED TO BE USED BY XILINX          
--                CUSTOMERS, AND WITHIN XILINX DEVICES.               
--                                                                     
--                Copyright (c) 2000 Xilinx, Inc.                      
--                All rights reserved                                  
--                                                                     
--                                                                      
--                                                                 
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;

entity SlaveSerial is
	port(
	
	-- Original Springboard Signals:
	
	ADDR 	: in 		STD_LOGIC_VECTOR(23 DOWNTO 0);
	DATA	: inout STD_LOGIC_VECTOR(15 DOWNTO 0);
	WEn  	: in 		STD_LOGIC;
	OEn 	: in 		STD_LOGIC;
	CS1n  : in 		STD_LOGIC;

	-- Virtex Configuration Signals:

	Virtex_CCLK 	  : out STD_LOGIC;
	
	Virtex_PROGRAM	: out STD_LOGIC;
	
	Virtex_DONE 	  : in  STD_LOGIC;
	Virtex_INIT 	  : in  STD_LOGIC;
	Virtex_Din    	: out STD_LOGIC

 );


end SlaveSerial;

architecture BEHAVE of SlaveSerial is

constant Configuration_Register_ADDR	                                       
	: STD_LOGIC_VECTOR(23 downto 0) := "000000000000000000000000";    
constant Program_Register_ADDR 			 
	: STD_LOGIC_VECTOR(23 downto 0) := "000000000000000000000010";
constant Input_Register_ADDR 				
	: STD_LOGIC_VECTOR(23 downto 0) := "000000000000000000000100";

signal Output_Buffer_Enable	: std_logic;	-- Output Buffer Enable Signal

signal input_reg_d1		: std_logic;
signal input_reg_d0		: std_logic;
signal input_register_data 	: std_logic_vector(15 DOWNTO 0);

signal Virtex_PROGRAM_OE	: std_logic;

begin



Output_Buffer_Enable <= not OEn and not CS1n;

DATA(15 DOWNTO 0) <= Input_Register_Data(15 DOWNTO 0) 
		when (Output_Buffer_Enable = '1') else (others => 'Z');

--Configuration Register

process(ADDR, DATA, CS1n, WEn)
begin
	if(WEn'event and WEn='1') then
		if(CS1n = '0') then
		  if (ADDR = Configuration_Register_ADDR) then
		  	Virtex_CCLK <= DATA(1);
			Virtex_Din  <= DATA(0);
		  end if;
		end if;
	end if;
end process;


--Program Register

process(ADDR, DATA, CS1n, WEn)
begin
	if(WEn'event and WEn='1') then
		if(CS1n = '0') then
		  if (ADDR = Program_Register_ADDR) then
			Virtex_PROGRAM_OE <= DATA(0);
		  end if;
		end if;
	end if;
end process;

Virtex_PROGRAM <= '0' when Virtex_PROGRAM_OE = '1' else 'Z';

--Input Register

process(ADDR, DATA, CS1n, OEn)
begin

	if(CS1n = '1') then
		if(OEn'event and OEn='0') then
		  if (ADDR = Input_Register_ADDR) then
			Input_reg_D1 		<= Virtex_DONE;
			Input_reg_D0		<= Virtex_INIT;
		  end if;
		end if;
	end if;
end process;

Input_Register_Data <= ("00000000000000" & Input_Reg_D1  & Input_Reg_D0);


END BEHAVE;
