#!/usr/local/bin/perl5
 
################################################################################ 
#
# Title : bitformat.pl
# Last Modified : September 9, 2001
#
# Description : This script will take in a bit file, run the Xilinx PROMGen 
#   program on it to get a hex type output, and then parse that output 
#   according to a set of variables, in order to format the data in a 
#   microprocessor friendly form.
# 
# Usage : bitformat.pl input_bitfile output_file
#
# Disclaimer : This Perl script is provided "as is" with no warranty
#   whatsoever and Xilinx specifically disclaims any implied warranties
#   of merchantability and fitness for a particular purpose, or against
#   infringement.  They are only intended to be used by Xilinx customers,
#   and within Xilinx designs.
#
# Copyright (c) 2001 Xilinx, Inc.
# All rights reserved
#
################################################################################ 

## Dictates whether to employ byte swapping (1) or not (0)
$byteSwap = 0;

## String that appears before data starts (can do multiple lines
## with the \n (newline) character)
$preDataString = 
"int *myrecord = {\n";

## String that appears after all the data is done (can do multiple
## lines with the \n (newline) character)
$postDataString = 
"}";

## String that appears before each data item on a data line
$dataLineBegin = 
"\t0x";

## String that appears after each data item on a data line
$dataLineEnd = ",";

## Number of half-bytes should be printed per data line
$nibblesPerLine = 4;

## Dictates whether to use a counter in the output.  If enabled, the address
## will appear at the beginning of each data line, before $dataLineBegin
$useAddressCounter = 0;

## Sets the inital address counter value (if used)
$addressCounter = 0;

## Sets the address counter output format (see the Perl 'sprintf' function for
## details)
$addressCounterFormat = "%6.6X";

################################################################################ 
################################################################################ 
## BEGIN PROGRAM BODY - DO NOT CHANGE
################################################################################ 
################################################################################ 

if ($#ARGV != 1) {
  print STDERR "usage : bitformat.pl input_bit_file output_file\n";
  exit(1);
}

if ($byteSwap == 0){
  $info = `promgen -w -b -p hex -o temp.hex -u 0 $ARGV[0]`;
} else {
  $info = `promgen -w -p hex -o temp.hex -u 0 $ARGV[0]`;
}

open (INFILE, "temp.hex") or die "Can't open file $ARGV[0]: $!\n";
open (OUTFILE, ">$ARGV[1]") or die "Can't open file $ARGV[1]: $!\n";

print OUTFILE "$preDataString";

$count = 0;
$line = "";
$c = getc INFILE;


while ($c =~ /[A-Fa-f0-9]/){
  if ($count == $nibblesPerLine) {
    $count = 0;
    if ($useAddressCounter == 1) {
      $address = sprintf $addressCounterFormat, $addressCounter;
      print "address = $address\n" if $debug;
      print OUTFILE "${address}${dataLineBegin}${line}${dataLineEnd}\n";
    } else {
      print OUTFILE "${dataLineBegin}${line}${dataLineEnd}\n";
    }
    $line = "";
    $addressCounter++;
  }
  $line = $line . uc($c);
  $count++;
  $c = getc INFILE;
}

###  Print last data entry
if ($useAddressCounter == 1){
  print OUTFILE "${address}${dataLineBegin}${line}${dataLineEnd}\n";
} else {
  print OUTFILE "${dataLineBegin}${line}${dataLineEnd}\n";
}  

print OUTFILE "$postDataString\n";

close OUTFILE;

################################################################################ 
################################################################################ 
## END PROGRAM BODY - DO NOT CHANGE
################################################################################ 
################################################################################ 

